/*
 * export.c
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gtk/gtk.h>
#include "interface.h"
#include "export.h"
#include "support.h"
#include "callbacks.h"
#include <xhtmlite/xhtmlite_module/xhtmlbuffer.h>

//for xml
#include <libxml/xmlreader.h>

//for transformation
#include <string.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

static gboolean byoj_export_answer_in_form_of_question = TRUE;
static GtkWidget *byoj_export_dialog;

#ifdef G_OS_WIN32
 #include <windows.h>
#endif

static gint byoj_open_exported_file(const char *filename)
{
	gint ret = 0;
 	#ifdef G_OS_WIN32
		//system(g_strdup_printf("%s > NULL", filename)); //locks up the application at this point
		ShellExecute(HWND_DESKTOP, "open", filename, NULL, NULL, SW_SHOWNORMAL); //works well for windows
	ret = 1;
	#else
		GError *error = NULL;

		gchar * commandname= g_strdup("sensible-browser");  //default
		#ifdef HAVE_GNOME
			commandname = g_strdup("gnome-open");
		#elif HAVE_KDE
			commandname = g_strdup("kfmclient exec");
		#endif
		gchar *command[] = {commandname, g_strdup (filename), NULL};
		ret = g_spawn_async (NULL, command, NULL, G_SPAWN_SEARCH_PATH,
                          NULL, NULL, NULL, &error);
		if(error)
		{
			printf("Error: Opening File with command '%s'\n", commandname);
		}
	#endif
	return ret;
}

static int byoj_export_is_directory(char *filename)
{
	size_t len= strlen(filename);
	int ret;

	if (filename[len-1]== '\\')
	{
		filename[len-1]= '\0';
		ret= g_file_test(filename,G_FILE_TEST_IS_DIR);
		filename[len-1]= '\\';
	}
	else
		ret= g_file_test(filename,G_FILE_TEST_IS_DIR);

	return ret;
}

//directory has to end with a backslash
static void byoj_export_get_all_filenames(const char *directory, GArray *fileList)
{
	GDir *dir;
	const char *str;

	if ( !(dir= g_dir_open(directory, 0, NULL)) )
	{
		fprintf(stderr, "Cannot open directory %s\n", directory);
		return;
	}

	while (str= g_dir_read_name(dir) )
	{
		char *new_dir = g_strconcat(directory,str,"\\",NULL);

		if (byoj_export_is_directory(new_dir))
		{
			byoj_export_get_all_filenames(new_dir, fileList);
		}
		else
		{
			if(g_strstr_len(str, strlen(str), ".xsl") != NULL)
			{
				char *fn = g_strconcat(directory,str, NULL);
				g_array_append_val(fileList,fn);
			}
		}

		g_free(new_dir);
	}
}

static void byoj_export_process_XSLT_Node(xmlTextReaderPtr reader, GtkTreeModel *store, GtkTreeIter iter)
{
	const xmlChar *name, *value;

	name = xmlTextReaderConstName(reader);
	if (name == NULL)
		name = BAD_CAST "--";

	value = xmlTextReaderConstValue(reader);

	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-name") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    0, value,
				    -1);
	}
	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-description") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    2, value,
				    -1);
	}
	if(!xmlStrcasecmp(name, (const xmlChar*)"xs:template-image") && xmlTextReaderNodeType(reader)== XML_READER_TYPE_ELEMENT  )
	{
		xmlTextReaderRead(reader);
		value = xmlTextReaderConstValue(reader);
		printf("gotcha:%s\n",value);
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    3, value,
				    -1);
	}
	printf("%d %d %s %d %d",
	       xmlTextReaderDepth(reader),
	       xmlTextReaderNodeType(reader),
	       name,
	       xmlTextReaderIsEmptyElement(reader),
	       xmlTextReaderHasValue(reader));
	if (value == NULL)
		printf("\n");
	else {
		if (xmlStrlen(value) > 40)
			printf(" %.40s...\n", value);
		else
			printf(" %s\n", value);
	}
}

static void byoj_export_read_XSLT_file(const char *filename, GtkTreeModel *store)
{
	xmlTextReaderPtr reader;
	int ret;

	reader = xmlReaderForFile(filename, NULL, 0);
	if (reader != NULL) {
		GtkTreeIter iter;
		gtk_tree_store_append (GTK_TREE_STORE(store), &iter, NULL);  /* Acquire an iterator */
		gtk_tree_store_set (GTK_TREE_STORE(store), &iter,
				    1, filename,
				    -1);
		
		ret = xmlTextReaderRead(reader);
		while (ret == 1) {
			byoj_export_process_XSLT_Node(reader, store, iter);
			ret = xmlTextReaderRead(reader);
		}
		xmlFreeTextReader(reader);
		if (ret != 0) {
			fprintf(stderr, "%s : failed to parse\n", filename);
		}
	} else {
		fprintf(stderr, "Unable to open %s\n", filename);
	}
}

static void
byoj_export_setup_XSLT_treeview (GtkWidget *treeview1)
{

	//create new treeview1 store and columns
	GtkTreeStore *store = gtk_tree_store_new (4,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING,
			G_TYPE_STRING); //name, filename, description, picture
	gtk_tree_view_set_model(GTK_TREE_VIEW(treeview1), GTK_TREE_MODEL(store));
	
	GtkCellRenderer *NameRender = gtk_cell_renderer_text_new ();
	g_object_set_data (G_OBJECT (NameRender), "column", 0);

	GtkTreeViewColumn *NameCol = gtk_tree_view_column_new ();
	gtk_tree_view_column_set_title(NameCol, "Templates");
	gtk_tree_view_column_pack_start (NameCol, NameRender, TRUE);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 0);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 1);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 2);
	gtk_tree_view_column_add_attribute (NameCol, NameRender, "text", 3);
	gtk_tree_view_column_set_expand (NameCol, FALSE);
	gtk_tree_view_column_set_resizable(NameCol, TRUE);
	gtk_tree_view_column_set_sort_column_id(NameCol, 0);
	gtk_tree_view_append_column (GTK_TREE_VIEW(treeview1), NameCol);

	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(treeview1), TRUE);
}

static void byoj_on_treeview_template_cursor_changed (GtkTreeView *treeview, gpointer user_data)
{
	//GtkWidget *treeview1 = lookup_widget(GTK_WIDGET(toolbutton), "treeview1");
	GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
	GtkTreeIter iter;
	if(gtk_tree_selection_get_selected (selection, &store, &iter))
	{
		//Set export button sensitive
		GtkWidget *byoj_export_button_ok = lookup_widget(GTK_WIDGET(treeview), "byoj_export_button_ok");
		gtk_widget_set_sensitive (byoj_export_button_ok, TRUE);
		//Fill Description box
		gchar *description, *xsl_filename;
		printf("Cursor Changed: Getting description\n");
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 2, &description, -1);
		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 1, &xsl_filename, -1);
		if(description!= NULL)
		{
			GtkTextView *textview_description = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(treeview), "byoj_textview_description"));
			GtkTextBuffer *textbuffer = gtk_text_view_get_buffer  (textview_description);
			printf("Cursor Changed: Importing HTML, Description(%s)\n XSL: FileName:%s\n", description, xsl_filename);
        		xhtmlite_import (textbuffer, description, TRUE);
		}
		else
		{
		}
	}
}

extern int xmlLoadExtDtdDefaultValue;
static int byoj_export_transform(xmlDocPtr doc, char *xsltFileName, char *OutputFileName)
{
	int i;
	const char *params[16 + 1];
	int nbparams = 0;
	xsltStylesheetPtr cur = NULL;
	xmlDocPtr res;
 
    params[nbparams++] = "screen_width";
    params[nbparams++] = "'800'";
    params[nbparams++] = "xsltFileName";
    params[nbparams++] = g_strdup_printf("'%s'", xsltFileName);
    params[nbparams++] = "OutputFileName";
    params[nbparams++] = g_strdup_printf("'%s'", OutputFileName);
    params[nbparams++] = "answer_in_form_of_question";
    params[nbparams++] = g_strdup_printf("'%d'", byoj_export_answer_in_form_of_question);
    params[nbparams++] = "ColumnHeightPercent";
    params[nbparams++] = g_strdup_printf("'%s'", OutputFileName);
    params[nbparams++] = "ColumnWidthPercent";
    params[nbparams++] = g_strdup_printf("'%s'", OutputFileName);
    params[nbparams] = NULL;

	printf("params:%d\n", nbparams);
	
	xmlSubstituteEntitiesDefault(1);
	xmlLoadExtDtdDefaultValue = 1;
	cur = xsltParseStylesheetFile((const xmlChar *)xsltFileName);
	i++;
	//xsltQuoteOneUserParam		(cur,
	//				 "answer_in_form_of_question",
	//				 "TRUE");
	
	//doc = xmlParseFile(xmlFileName);
	res = xsltApplyStylesheet(cur, doc, params);
	xsltSaveResultToFile(stdout, res, cur);
	xsltSaveResultToFilename(OutputFileName, res, cur, 0);
	xsltFreeStylesheet(cur);
	xmlFreeDoc(res);
	xmlFreeDoc(doc);

	xsltCleanupGlobals();
	xmlCleanupParser();
	return 1;
}

void byoj_export_dialog_open (GtkWidget *widget)
{
	if(byoj_export_dialog == NULL)
	{
		byoj_export_dialog = create_byoj_dialog_export ();

		GtkWidget *treeview1 = lookup_widget(byoj_export_dialog, "byoj_treeview_template");
		byoj_export_setup_XSLT_treeview(treeview1);
		GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview1));
	
		GtkTextView *textview_description = GTK_TEXT_VIEW(lookup_widget(GTK_WIDGET(byoj_export_dialog), "byoj_textview_description"));
		GtkTextBuffer *textbuffer = xhtmlite_create_buffer(textview_description, FALSE);
        //for templates
        #ifdef G_OS_WIN32
               gchar *package_executable_dir = g_win32_get_package_installation_directory (NULL, NULL);
        #else
               gchar *package_executable_dir = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, NULL);
        #endif
		const gchar *export_dir_string;
		export_dir_string = g_build_filename (package_executable_dir, "export/", NULL);
		printf("%s\n", export_dir_string);
		GArray *file_list;
		file_list= g_array_new(0,1,sizeof(char *));
		byoj_export_get_all_filenames(export_dir_string, file_list);
		
		int i;
		for (i=0; (guint)i < file_list->len; i++) {
			char *fn= g_array_index(file_list, char *, i);
			printf("+ %s\n", fn);
			byoj_export_read_XSLT_file(fn, store);
	
			g_free(fn);
		}
	
		g_signal_connect ((gpointer) treeview1, "cursor_changed",
					G_CALLBACK (byoj_on_treeview_template_cursor_changed),
					NULL);
		g_array_free(file_list, 0);
	}
	    //Set already selected answer format
		GtkWidget *byoj_export_radiobutton_answer_in_form_of_question = lookup_widget(GTK_WIDGET(byoj_export_dialog), "byoj_export_radiobutton_answer_in_form_of_question");
		if(byoj_answer_in_form_of_question)
		{
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(byoj_export_radiobutton_answer_in_form_of_question), TRUE);
        }
        else
        {
            GtkWidget *byoj_export_radiobutton_answers_the_question = lookup_widget(GTK_WIDGET(byoj_export_dialog), "byoj_export_radiobutton_answers_the_question");         
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(byoj_export_radiobutton_answers_the_question), TRUE); 
        }
	if (gtk_dialog_run (GTK_DIALOG (byoj_export_dialog)) == GTK_RESPONSE_OK)
	{
	   	gtk_widget_hide(byoj_export_dialog);
        byoj_export_answer_in_form_of_question = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(byoj_export_radiobutton_answer_in_form_of_question));
    	GtkWidget *byoj_treeview_template = lookup_widget(byoj_export_dialog, "byoj_treeview_template");
        GtkTreeModel *store = gtk_tree_view_get_model(GTK_TREE_VIEW(byoj_treeview_template));
		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(byoj_treeview_template));
		GtkTreeIter iter_file;
		if(gtk_tree_selection_get_selected (selection, &store, &iter_file))
		{
		//Fill Description box
			gchar *xslt_filename;
			gtk_tree_model_get (GTK_TREE_MODEL(store), &iter_file, 1, &xslt_filename, -1);
			if(xslt_filename!= NULL)
			{
				GtkWidget *dialog;
				dialog = create_byoj_filechooserdialog ();
				GtkWidget *btnOK = lookup_widget(GTK_WIDGET(dialog), "btnOK");
				gtk_button_set_label (GTK_BUTTON(btnOK), "gtk-save");
				gtk_button_set_use_stock (GTK_BUTTON(btnOK), TRUE);
				gtk_file_chooser_set_action(GTK_FILE_CHOOSER (dialog), GTK_FILE_CHOOSER_ACTION_SAVE);
				gtk_window_set_title(GTK_WINDOW(dialog), "Save File");

				if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
				{
					char *export_filename;
					export_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));

					GtkWidget *byoj_treeview = lookup_widget(widget, "byoj_treeview");
					xmlDocPtr doc = byoj_write_xmldocptr(byoj_treeview);
					if(byoj_export_transform(doc,xslt_filename,export_filename))
					{
						byoj_open_exported_file(export_filename);
					}
					g_free(export_filename);
				}
				gtk_widget_destroy (dialog);
			}
		}
	}
	gtk_widget_hide(byoj_export_dialog);
	
}
